/**
 * Code released under WTFPL [http://sam.zoy.org/wtfpl/]
 * 
 * @author Arnaud Pichery [http://coderpeon.ovh.org]
 */
package com.flexspy
{
	import flash.display.DisplayObject;
	import flash.events.Event;
	import flash.utils.describeType;
	
	import mx.collections.ArrayCollection;
	import mx.collections.Sort;
	import mx.collections.SortField;
	import mx.containers.HBox;
	import mx.containers.Panel;
	import mx.controls.Alert;
	import mx.controls.DataGrid;
	import mx.controls.TextInput;
	import mx.controls.dataGridClasses.DataGridColumn;
	import mx.core.UIComponent;
	import mx.events.CollectionEvent;
	import mx.managers.PopUpManager;
	import mx.controls.Text;
	import mx.utils.ObjectUtil;
	import flash.utils.getQualifiedClassName;
	import mx.containers.Canvas;
	import mx.containers.VBox;
	
	public class ComponentProperties extends VBox {

		private var _unfilteredComponentProperties: Array;
		private var _componentProperties: ArrayCollection = new ArrayCollection();
		private var _componentTable: DataGrid;
		private var _filter: String;
		
	    /**
	     * Initializes a new instance of this class
	     */
		public function ComponentProperties() {
			super();
	
			// properties
			this.label = "Properties";
			this.setStyle("paddingLeft", 10);
			this.setStyle("paddingRight", 10);
			this.setStyle("paddingBottom", 10);
		}
		
	    /**
	     * @private
	     **/
		override protected function createChildren():void {
			super.createChildren();
			
			// Filter box
			var filterBox: mx.containers.HBox = new mx.containers.HBox();
			
			var uiText: mx.controls.Text = new mx.controls.Text();
			uiText.text = "Filter";
			filterBox.addChild(uiText);
			
			var filterTextInput: TextInput = new mx.controls.TextInput();
			filterTextInput.addEventListener("change", updateFilter);
			filterBox.addChild(filterTextInput);
			this.addChild(filterBox);

			// Component table			
			var col1: mx.controls.dataGridClasses.DataGridColumn = new mx.controls.dataGridClasses.DataGridColumn();
			col1.width = 160;
			col1.headerText = "Attribute";
			col1.dataField = "name";

			var col2: mx.controls.dataGridClasses.DataGridColumn = new mx.controls.dataGridClasses.DataGridColumn();
			col2.headerText = "Value";
			col2.dataField = "value";
			col2.dataTipFunction = showValueDataTip;
			col2.showDataTips = true;

			_componentTable = new mx.controls.DataGrid();
			_componentTable.percentWidth = 100.0;
			_componentTable.percentHeight = 100.0;
			_componentTable.columns = [col1, col2];
			this.addChild(_componentTable);
		}
		
		private function showValueDataTip(obj: Object): String {
			var result: String = "";
			var objValue: Object = obj["value"];
			try {
				result = String(objValue); //ObjectUtil.toString(objValue);
			} catch (err: Error) {
			}
			var strValue: String = String(objValue);
			if (strValue == result) {
				result = ""; // Do not display any tooltip if it's identical to the attribute value.
			}
			return result;
		}
		
        public function showArrayProperties(array: Array): void {
        	if(array != null){
	        	// First remove all fields.
				_unfilteredComponentProperties = new Array();
	        	for(var i:int = 0; i<array.length;i++){
	        		addObjectProperties(i, array[i]);
	        	}        		
        	}
        }
        
        public function showArrayCollectionProperties(array: ArrayCollection): void {
        	if(array != null){
	        	// First remove all fields.
				_unfilteredComponentProperties = new Array();
			
	        	for(var i:int = 0; i<array.length;i++){
	        		addObjectProperties(i, array.getItemAt(i));
	        	}        		
			
				updateTableContent();
        	}
        }
        
        public function showComponentProperties(displayObject: Object): void {
        	// First remove all fields.
			_unfilteredComponentProperties = new Array();
			
        	addObjectProperties(-1, displayObject);
			
			updateTableContent();
        }
        private function addObjectProperties(index: int, displayObject: Object): void {
        	
        	if (displayObject != null) {
				var description: XML = describeType(displayObject);
				
				var attributeList: Array = new Array();
				var property: Object;
				
				for each (var accessor: XML in description.accessor) {
					property = inspectXMLProperty(index, accessor, displayObject, attributeList);
					if (property != null) {
						attributeList.push(property.name);
						_unfilteredComponentProperties.push(property);
					}
				}
				for each (var variable: XML in description.variable) {
					property = inspectXMLProperty(index, variable, displayObject, attributeList);
					if (property != null) {
						attributeList.push(property.name);
						_unfilteredComponentProperties.push(property);
					}
				}
				for(var name:String in displayObject) {
					property = inspectProperty(index, name, getQualifiedClassName(displayObject[name]), displayObject, attributeList);
					if (property != null) {
						attributeList.push(property.name);
						_unfilteredComponentProperties.push(property);
					}					
				}
        	}
        }
        
        private function filterList(source: Array, filter: String): Array {
        	var filteredArray: Array = new Array();
        	var pattern: String = (filter == null || filter.length == 0) ? null : filter.toLocaleLowerCase();
        	
        	for each (var property: Object in source) {
        		var propName: String = property["name"];
        		if (pattern == null || propName.toLocaleLowerCase().indexOf(pattern) >= 0) {
        			filteredArray.push(property);
        		}
        	}
        	return filteredArray;
        }
        
        private function inspectXMLProperty(index: int, property: XML, displayObject: Object, attributeList: Array): Object {
			trace("inspecting property " + property.@name);
			if (property.@access == "writeonly") {
				return null;
			}
			return inspectProperty(index, property.@name, property.@type, displayObject, attributeList);
        }
        
        private function inspectProperty(index: int, name: String, type: String, displayObject: Object, attributeList: Array): Object {
			if (name == null || name.length == 0 || name.charAt(0) == '$' || attributeList.indexOf(name) >= 0) {
				// Invalid or private property, or already present (describeType method might return several times the same properties)
				return null;
			}
			var attributeValue: Object
			//attributeValue = String(displayObject[name]); // THIS LINE MAKES THE FLASH PLAYER 9 CRASH !!
			if (type == "Number" || type == "String" || type == "Boolean" || type == "int" || type == "uint" || type == "Date") {
				try {
					attributeValue = displayObject[name];
				} catch (error: ReferenceError) {
					// Unable to retrieve property value.
					attributeValue = "[Unable to retrieve value]";
					trace(attributeValue + error.message);
				}
			} else {
				attributeValue = "[" + type + "]";
			}

			var attribute: Object = new Object();
			if(index != -1){
				name = "["+ index + "]." + name;
			}
			attribute["name"] = name;
			attribute["value"] = attributeValue;
			return attribute;
        }
        
        private function updateFilter(event: Event): void {
        	_filter = mx.controls.TextInput(event.target).text;
        	updateTableContent();
        }
        
        private function updateTableContent(): void {
			var array: Array = filterList(_unfilteredComponentProperties, _filter);
			var collection: ArrayCollection = new ArrayCollection(array);
			var oldCollection: ArrayCollection = _componentTable.dataProvider as ArrayCollection;
			if (oldCollection != null) {
				collection.sort = oldCollection.sort;
			} else {
				var sort: Sort = new Sort();
				sort.fields = [ new SortField("name", true) ]; 
				collection.sort = sort;
			}
			collection.refresh();
			_componentTable.dataProvider = collection;
        }
	}
}