/**
 * Code released under WTFPL [http://sam.zoy.org/wtfpl/]
 * 
 * @author Arnaud Pichery [http://coderpeon.ovh.org]
 */
package com.flexspy
{
	import flash.display.DisplayObject;
	import flash.events.Event;
	import flash.utils.describeType;
	
	import mx.collections.ArrayCollection;
	import mx.collections.Sort;
	import mx.collections.SortField;
	import mx.containers.HBox;
	import mx.containers.Panel;
	import mx.controls.Alert;
	import mx.controls.DataGrid;
	import mx.controls.TextInput;
	import mx.controls.dataGridClasses.DataGridColumn;
	import mx.core.UIComponent;
	import mx.events.CollectionEvent;
	import mx.managers.PopUpManager;
	import mx.controls.Text;
	import mx.utils.ObjectUtil;
	import flash.utils.getQualifiedClassName;
	import mx.containers.Canvas;
	import mx.containers.VBox;
	import mx.styles.CSSStyleDeclaration;
	
	public class ComponentStyles extends VBox {
		private static var COLOR: String = "color";
		private static var COLORS: String = "colors";

		private var _unfilteredComponentProperties: Array;
		private var _componentProperties: ArrayCollection = new ArrayCollection();
		private var _componentTable: DataGrid;
		private var _filter: String;
		
		
	    /**
	     * Initializes a new instance of this class
	     */
		public function ComponentStyles() {
			super();
	
			// properties
			this.label = "Styles";
			this.setStyle("paddingLeft", 10);
			this.setStyle("paddingRight", 10);
			this.setStyle("paddingBottom", 10);
		}
		
	    /**
	     * @private
	     **/
		override protected function createChildren():void {
			super.createChildren();
			
			// Filter box
			var filterBox: mx.containers.HBox = new mx.containers.HBox();
			
			var uiText: mx.controls.Text = new mx.controls.Text();
			uiText.text = "Filter";
			filterBox.addChild(uiText);
			
			var filterTextInput: TextInput = new mx.controls.TextInput();
			filterTextInput.addEventListener("change", updateFilter);
			filterBox.addChild(filterTextInput);
			this.addChild(filterBox);

			// Component table			
			var col1: mx.controls.dataGridClasses.DataGridColumn = new mx.controls.dataGridClasses.DataGridColumn();
			col1.width = 160;
			col1.headerText = "Style";
			col1.dataField = "name";

			var col2: mx.controls.dataGridClasses.DataGridColumn = new mx.controls.dataGridClasses.DataGridColumn();
			col2.headerText = "Value";
			col2.dataField = "value";

			_componentTable = new mx.controls.DataGrid();
			_componentTable.percentWidth = 100.0;
			_componentTable.percentHeight = 100.0;
			_componentTable.columns = [col1, col2];
			this.addChild(_componentTable);
		}
		
        public function showComponentStyles(displayObject: Object): void {
        	// First remove all fields.
			_unfilteredComponentProperties = new Array();
        	if (displayObject != null) {
	        	addObjectProperties(displayObject);
	        }
			updateTableContent();
        }
        
        private function addObjectProperties(displayObject: Object): void {
			var attributeList: Array = new Array();
			var property: Object;
			var style: String;

			var nonInheritingStyles: Object = displayObject["nonInheritingStyles"];
			if (nonInheritingStyles != null) {
				for (style in nonInheritingStyles) {
					property = inspectStyle(style, nonInheritingStyles, attributeList);
					if (property != null) {
						attributeList.push(property.name);
						_unfilteredComponentProperties.push(property);
					}
				}
			}

			var inheritingStyles: Object = displayObject["inheritingStyles"];
			if (inheritingStyles != null) {
				for (style in inheritingStyles) {
					property = inspectStyle(style, inheritingStyles, attributeList);
					if (property != null) {
						attributeList.push(property.name);
						_unfilteredComponentProperties.push(property);
					}
				}
			}
        }
        
        private function filterList(source: Array, filter: String): Array {
        	var filteredArray: Array = new Array();
        	var pattern: String = (filter == null || filter.length == 0) ? null : filter.toLocaleLowerCase();
        	
        	for each (var property: Object in source) {
        		var propName: String = property["name"];
        		if (pattern == null || propName.toLocaleLowerCase().indexOf(pattern) >= 0) {
        			filteredArray.push(property);
        		}
        	}
        	return filteredArray;
        }
        
        private function inspectStyle(name: String, styles: Object, attributeList: Array): Object {
			trace("inspecting style " + name);

			if (name == null || name.length == 0 || name.charAt(0) == '$' || attributeList.indexOf(name) >= 0) {
				// Invalid or private style, or already present (method might return several times the same properties)
				return null;
			}

			var attribute: Object = new Object();
			var attributeValue: Object = styles[name];
			attribute["name"] = name;
			attribute["value"] = attributeValue;

			// Attribute whose name ends with "color(s)" are printed in hexa.
			if ((attributeValue is Number) && (name.length >= COLOR.length) && (name.substr(name.length - COLOR.length, COLOR.length).toLowerCase() == COLOR)) {
				attribute["value"] = toHexColor(Number(attributeValue));
			}
			if ((attributeValue is Array) && (name.length >= COLORS.length) && (name.substr(name.length - COLORS.length, COLORS.length).toLowerCase() == COLORS)) {
				var attributeValues: Array = attributeValue as Array;
				for (var i: int = 0; i < attributeValues.length; i++) {
					if (attributeValues[i] is Number) {
						attributeValues[i] = toHexColor(Number(attributeValues[i]));
					}
				}
				attribute["value"] = attributeValues.join(", ");
			}
			return attribute;
        }
        
        private function updateFilter(event: Event): void {
        	_filter = mx.controls.TextInput(event.target).text;
        	updateTableContent();
        }
        
        private function updateTableContent(): void {
			var array: Array = filterList(_unfilteredComponentProperties, _filter);
			var collection: ArrayCollection = new ArrayCollection(array);
			var oldCollection: ArrayCollection = _componentTable.dataProvider as ArrayCollection;
			if (oldCollection != null) {
				collection.sort = oldCollection.sort;
			} else {
				var sort: Sort = new Sort();
				sort.fields = [ new SortField("name", true) ]; 
				collection.sort = sort;
			}
			collection.refresh();
			_componentTable.dataProvider = collection;
        }
        
        private static function toHexColor(n: Number): String {
        	var result: String = n.toString(16);
        	while (result.length < 6) {
        		result = "0" + result;
        	}
        	return ("#" + result);
        }
	}
}